#include <kernel/idt.h>
#include <kernel/vga.h>
#include <kernel/printf.h>
#include <kernel/gdt.h>
#include <kernel/pic8259.h>

void encode_idt_entry(struct idt_entry idt[], int i, uint32_t offset, uint16_t segment_selector, uint8_t attributes) {
   idt[i].offset_lo = (uint16_t)(offset & 0xFFFF);
   idt[i].seg_select = segment_selector;
   idt[i].reserved = 0;
   idt[i].attributes = attributes;
   idt[i].offset_hi = (uint16_t)(offset >> 16);
}

extern uint32_t isr_stub_table[32];
extern void ps2_isr();
extern void syscall_isr();
void interrupts_init(struct idt_entry idt[], struct idtr idtr) {
    for (int i = 0; i < 32; i++) {
        encode_idt_entry(idt, i, isr_stub_table[i], GDT_SEGMENT_SELECTOR(GDT_KERNEL_CODE, 0x00), TRAP_GATE_32 | INT_RING0 | INT_PRESENT);
    }

    encode_idt_entry(idt, 0x21, (uint32_t)&ps2_isr, GDT_SEGMENT_SELECTOR(GDT_KERNEL_CODE, RING0), INT_GATE_32 | INT_RING0 | INT_PRESENT);
    encode_idt_entry(idt, 0x80, (uint32_t)&syscall_isr, GDT_SEGMENT_SELECTOR(GDT_USER_CODE, RING3), INT_GATE_32 | INT_RING3 | INT_PRESENT);

    idtr.size = (sizeof(struct idt_entry) * 256) - 1;
    idtr.base = (uint32_t) idt;

    asm volatile ("lidt %0" :: "m"(idtr));

    pic_remap(PIC_1_START, PIC_2_START);

    asm volatile ("sti" ::);
}

void exception_handler(struct registers regs) {
    clear_screen();

    printf("Exception 0x%X (0x%X) has occurred\nEAX: 0x%08X\nECX: 0x%08X\nEDX: 0x%08X\nEBX: 0x%08X\nESI: 0x%08X\nEDI: 0x%08X\nEBP: 0x%08X\nESP: 0x%08X\n",
            regs.int_no, regs.err_code, regs.eax, regs.ecx, regs.edx, regs.ebx, regs.esi, regs.edi, regs.ebp, regs.esp+24); 

    printf("\nEIP:     0x%08X\nCS:      0x%08X\nEFLAGS:  0x%08X",
            regs.eip, regs.cs, regs.eflags);
}
